# This file was auto-generated by Fern from our API Definition.

import typing

LlmAsJudgeMessagePublicRole = typing.Union[
    typing.Literal["SYSTEM", "USER", "AI", "TOOL_EXECUTION_RESULT"], typing.Any
]
